@@ -15,7 +15,6 @@
 use bounded_vec_deque::BoundedVecDeque;
 use serde::{Deserialize, Serialize};
 use std::collections::{BTreeMap, HashMap};
-use std::convert::TryInto;
 use std::time::Duration;
 use tor_netdir::params::NetParameters;
 
@@ -266,7 +265,7 @@ impl History {
     fn estimate_xm(&self, n_modes: usize) -> Option<u32> {
         // From path-spec:
         //   Tor clients compute the Xm parameter using the weighted
-        //   average of the the midpoints of the 'cbtnummodes' (10)
+        //   average of the midpoints of the 'cbtnummodes' (10)
         //   most frequently occurring 10ms histogram bins.
 
         // The most frequently used bins.
@@ -580,8 +579,13 @@ impl super::TimeoutEstimator for ParetoTimeoutEstimator {
     fn note_circ_timeout(&mut self, hop: u8, delay: Duration) {
         // Only record this timeout if we have seen some network activity since
         // we launched the circuit.
-        let last_traffic = tor_proto::time_since_last_incoming_traffic();
-        let have_seen_recent_activity = last_traffic < delay;
+        let have_seen_recent_activity =
+            if let Some(last_traffic) = tor_proto::time_since_last_incoming_traffic() {
+                last_traffic < delay
+            } else {
+                // TODO: Is this the correct behavior in this case?
+                true
+            };
 
         if hop > 0 && have_seen_recent_activity {
             self.history.add_success(false);
